from allotropy.allotrope.models.pcr_benchling_2023_09_qpcr import (
    BaselineCorrectedReporterDataCube,
    CalculatedDataDocumentItem,
    ContainerType,
    DataProcessingDocument,
    DataProcessingDocument1,
    DataSourceAggregateDocument,
    DataSourceDocumentItem,
    DataSystemDocument,
    DeviceControlAggregateDocument,
    DeviceControlDocumentItem,
    DeviceSystemDocument,
    ExperimentType,
    MeasurementAggregateDocument,
    MeasurementDocumentItem,
    MeltingCurveDataCube,
    Model,
    NormalizedReporterDataCube,
    PassiveReferenceDyeDataCube,
    ProcessedDataAggregateDocument,
    ProcessedDataDocumentItem,
    QPCRAggregateDocument,
    QPCRDocumentItem,
    ReporterDyeDataCube,
    SampleDocument,
    TCalculatedDataAggregateDocument,
)
from allotropy.allotrope.models.shared.definitions.custom import (
    TNullableQuantityValueUnitless,
    TQuantityValueNumber,
    TQuantityValueUnitless,
)
from allotropy.allotrope.models.shared.definitions.definitions import (
    FieldComponentDatatype,
    TDatacubeComponent,
    TDatacubeData,
    TDatacubeStructure,
)
from allotropy.constants import ASM_CONVERTER_NAME, ASM_CONVERTER_VERSION
from allotropy.parsers.appbio_quantstudio.appbio_quantstudio_structure import (
    AmplificationData,
    Data,
    Header,
    MeltCurveRawData,
    MulticomponentData,
    RawData,
    Result,
    Well,
    WellItem,
    WellList,
)
from allotropy.parsers.appbio_quantstudio.calculated_document import (
    CalculatedDocument,
    DataSource,
)


def get_data() -> Data:
    well = Well(
        identifier=1,
        items={
            "IPC": WellItem(
                uuid="d170f2df-7ff8-4978-b13d-a7ad19fab824",
                identifier=1,
                position="A1",
                target_dna_description="IPC",
                sample_identifier="NAC",
                well_location_identifier="A1",
                reporter_dye_setting="VIC",
                quencher_dye_setting="NFQ-MGB",
                sample_role_type="BlockedIPC",
                amplification_data_obj=AmplificationData(
                    total_cycle_number_setting=1,
                    cycle=[1],
                    rn=[1.064],
                    delta_rn=[-0.002],
                ),
                result_obj=Result(
                    cycle_threshold_value_setting=0.2,
                    cycle_threshold_result=None,
                    automatic_cycle_threshold_enabled_setting=False,
                    automatic_baseline_determination_enabled_setting=False,
                    normalized_reporter_result=1.13,
                    baseline_corrected_reporter_result=None,
                    genotyping_determination_result="Blocked IPC Control",
                    genotyping_determination_method_setting=0.0,
                    quantity=None,
                    quantity_mean=None,
                    quantity_sd=None,
                    ct_mean=None,
                    ct_sd=None,
                    delta_ct_mean=None,
                    delta_ct_se=None,
                    delta_delta_ct=None,
                    rq=None,
                    rq_min=None,
                    rq_max=None,
                    rn_mean=1.261,
                    rn_sd=0.088,
                    y_intercept=None,
                    r_squared=None,
                    slope=None,
                    efficiency=None,
                ),
            ),
            "TGFb": WellItem(
                uuid="ba281c64-0605-4e76-8e9c-2a183be3cc08",
                identifier=1,
                position="A1",
                target_dna_description="TGFb",
                sample_identifier="NAC",
                well_location_identifier="A1",
                reporter_dye_setting="FAM",
                quencher_dye_setting="NFQ-MGB",
                sample_role_type="NTC",
                amplification_data_obj=AmplificationData(
                    total_cycle_number_setting=1,
                    cycle=[1],
                    rn=[0.343],
                    delta_rn=[-0.007],
                ),
                result_obj=Result(
                    cycle_threshold_value_setting=0.2,
                    cycle_threshold_result=None,
                    automatic_cycle_threshold_enabled_setting=False,
                    automatic_baseline_determination_enabled_setting=False,
                    normalized_reporter_result=0.402,
                    baseline_corrected_reporter_result=None,
                    genotyping_determination_result="Negative Control",
                    genotyping_determination_method_setting=0.0,
                    quantity=None,
                    quantity_mean=None,
                    quantity_sd=None,
                    ct_mean=None,
                    ct_sd=None,
                    delta_ct_mean=None,
                    delta_ct_se=None,
                    delta_delta_ct=None,
                    rq=None,
                    rq_min=None,
                    rq_max=None,
                    rn_mean=0.397,
                    rn_sd=0.006,
                    y_intercept=None,
                    r_squared=None,
                    slope=None,
                    efficiency=None,
                ),
            ),
        },
        multicomponent_data=MulticomponentData(
            cycle=[1],
            columns={
                "FAM": [502840.900],
                "ROX": [1591197.500],
                "VIC": [1654662.500],
            },
        ),
        melt_curve_raw_data=None,
        calculated_document=None,
    )

    return Data(
        header=Header(
            measurement_time="2010-10-01 01:44:54 AM EDT",
            plate_well_count=96,
            barcode=None,
            device_identifier="278880034",
            model_number="QuantStudio(TM) 6 Flex System",
            device_serial_number="278880034",
            analyst=None,
            experimental_data_identifier="QuantStudio 96-Well Presence-Absence Example",
            experiment_type=ExperimentType.presence_absence_qPCR_experiment,
            measurement_method_identifier="Ct",
            pcr_detection_chemistry="TAQMAN",
            passive_reference_dye_setting="ROX",
        ),
        wells=WellList([well]),
        raw_data=RawData(
            [
                "Well\tWell Position\tCycle\tx1-m1\tx2-m2\tx3-m3\tx4-m4\tx5-m5",
                "1\t      A1\t1\t882,830.500\t1,748,809.500\t1,648,195.400\t1,513,508.200\t3,796.012",
                "",
                "",
            ]
        ),
        endogenous_control="",
        reference_sample="",
        calculated_documents=[
            CalculatedDocument(
                uuid="d006e7e7-fbe4-47cf-821b-904e85202803",
                name="rn mean",
                value=1.261,
                iterated=True,
                data_sources=[
                    DataSource(
                        feature="normalized reporter result",
                        reference=well.items["IPC"],
                    ),
                ],
            ),
            CalculatedDocument(
                uuid="f4fee39c-5861-4203-afcf-94ee755ac0b4",
                name="rn sd",
                value=0.088,
                iterated=True,
                data_sources=[
                    DataSource(
                        feature="normalized reporter result",
                        reference=well.items["IPC"],
                    ),
                ],
            ),
            CalculatedDocument(
                uuid="e6707b0c-4494-412f-8a8e-ef51d01f25b3",
                name="rn mean",
                value=0.397,
                iterated=True,
                data_sources=[
                    DataSource(
                        feature="normalized reporter result",
                        reference=well.items["TGFb"],
                    ),
                ],
            ),
            CalculatedDocument(
                uuid="da78a225-2c34-40b3-b487-97893bfe491a",
                name="rn sd",
                value=0.006,
                iterated=True,
                data_sources=[
                    DataSource(
                        feature="normalized reporter result",
                        reference=well.items["TGFb"],
                    ),
                ],
            ),
        ],
    )


def get_data2() -> Data:
    well = Well(
        identifier=1,
        items={
            "B2M-Qiagen": WellItem(
                uuid="1e572828d-45fd-49bd-ab66-2a7f06aea3b6",
                identifier=1,
                position="A1",
                target_dna_description="B2M-Qiagen",
                sample_identifier="1. 200217 U251p14_-ab_-SEMA3F_8h_pA_1",
                well_location_identifier="A1",
                reporter_dye_setting="SYBR",
                quencher_dye_setting=None,
                sample_role_type="UNKNOWN",
                amplification_data_obj=AmplificationData(
                    total_cycle_number_setting=1,
                    cycle=[1],
                    rn=[0.612],
                    delta_rn=[-0.007],
                ),
                result_obj=Result(
                    cycle_threshold_value_setting=0.277,
                    cycle_threshold_result=18.717,
                    automatic_cycle_threshold_enabled_setting=True,
                    automatic_baseline_determination_enabled_setting=True,
                    normalized_reporter_result=None,
                    baseline_corrected_reporter_result=None,
                    genotyping_determination_result=None,
                    genotyping_determination_method_setting=None,
                    quantity=None,
                    quantity_mean=None,
                    quantity_sd=None,
                    ct_mean=18.717,
                    ct_sd=None,
                    delta_ct_mean=None,
                    delta_ct_se=None,
                    delta_delta_ct=None,
                    rq=None,
                    rq_min=None,
                    rq_max=None,
                    rn_mean=None,
                    rn_sd=None,
                    y_intercept=None,
                    r_squared=None,
                    slope=None,
                    efficiency=None,
                ),
            ),
        },
        multicomponent_data=MulticomponentData(
            cycle=[1],
            columns={
                "ROX": [55573.94],
                "SYBR": [34014.32],
            },
        ),
        melt_curve_raw_data=MeltCurveRawData(
            reading=[1],
            fluorescence=[3.478],
            derivative=[0.093],
        ),
    )

    return Data(
        header=Header(
            measurement_time="2001-12-31 09:09:19 PM EST",
            plate_well_count=384,
            barcode=None,
            device_identifier="278880086",
            model_number="ViiA 7",
            device_serial_number="278880086",
            analyst=None,
            experimental_data_identifier="200224 U251p14 200217_14v9_SEMA3F_trial 1",
            experiment_type=ExperimentType.comparative_CT_qPCR_experiment,
            measurement_method_identifier="Ct",
            pcr_detection_chemistry="SYBR_GREEN",
            passive_reference_dye_setting="ROX",
        ),
        wells=WellList([well]),
        raw_data=RawData(
            [
                "Well\tWell Position\tCycle\tx1-m1\tx2-m2\tx3-m3\tx4-m4\tx5-m5",
                "1\t      A1\t1\t36,431.130\t4,790.476\t4,560.821\t56,089.960\t-387.130",
                "",
                "",
            ]
        ),
        endogenous_control="",
        reference_sample="",
        calculated_documents=[],
    )


def get_model() -> Model:
    return Model(
        qPCR_aggregate_document=QPCRAggregateDocument(
            device_system_document=DeviceSystemDocument(
                device_identifier="278880034",
                model_number="QuantStudio(TM) 6 Flex System",
                device_serial_number="278880034",
                asset_management_identifier=None,
                firmware_version=None,
                description=None,
                brand_name=None,
                product_manufacturer=None,
            ),
            qPCR_document=[
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=96,
                            unit="#",
                            field_type=None,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="d170f2df-7ff8-4978-b13d-a7ad19fab824",
                                measurement_time="2010-10-01T01:44:54-04:00",
                                target_DNA_description="IPC",
                                sample_document=SampleDocument(
                                    sample_identifier="NAC",
                                    batch_identifier=None,
                                    sample_role_type="BlockedIPC",
                                    well_location_identifier="A1",
                                    well_plate_identifier=None,
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="VIC",
                                            quencher_dye_setting="NFQ-MGB",
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.2,
                                                    unit="(unitless)",
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=False,
                                                automatic_baseline_determination_enabled_setting=False,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=TQuantityValueUnitless(
                                                    value=0.0,
                                                    unit="(unitless)",
                                                    field_type=None,
                                                ),
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=None,
                                                unit="(unitless)",
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=TQuantityValueUnitless(
                                                value=1.13,
                                                unit="(unitless)",
                                                field_type=None,
                                            ),
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[1.064]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[-0.002]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result="Blocked IPC Control",
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=ReporterDyeDataCube(
                                    label="reporter dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="reporter dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[1654662.500]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                passive_reference_dye_data_cube=PassiveReferenceDyeDataCube(
                                    label="passive reference dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="passive reference dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[1591197.500]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                melting_curve_data_cube=None,
                            ),
                            MeasurementDocumentItem(
                                measurement_identifier="ba281c64-0605-4e76-8e9c-2a183be3cc08",
                                measurement_time="2010-10-01T01:44:54-04:00",
                                target_DNA_description="TGFb",
                                sample_document=SampleDocument(
                                    sample_identifier="NAC",
                                    batch_identifier=None,
                                    sample_role_type="NTC",
                                    well_location_identifier="A1",
                                    well_plate_identifier=None,
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="FAM",
                                            quencher_dye_setting="NFQ-MGB",
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.2,
                                                    unit="(unitless)",
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=False,
                                                automatic_baseline_determination_enabled_setting=False,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=TQuantityValueUnitless(
                                                    value=0.0,
                                                    unit="(unitless)",
                                                    field_type=None,
                                                ),
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=None,
                                                unit="(unitless)",
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=TQuantityValueUnitless(
                                                value=0.402,
                                                unit="(unitless)",
                                                field_type=None,
                                            ),
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.343]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[-0.007]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result="Negative Control",
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=ReporterDyeDataCube(
                                    label="reporter dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="reporter dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[502840.900]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                passive_reference_dye_data_cube=PassiveReferenceDyeDataCube(
                                    label="passive reference dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="passive reference dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[1591197.500]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                melting_curve_data_cube=None,
                            ),
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="QuantStudio 96-Well Presence-Absence Example",
                        experiment_type=ExperimentType.presence_absence_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                        well_volume=None,
                    ),
                    analyst=None,
                    submitter=None,
                )
            ],
            data_system_document=DataSystemDocument(
                data_system_instance_identifier="localhost",
                file_name="appbio_quantstudio_test01.txt",
                UNC_path="",
                software_name="Thermo QuantStudio",
                software_version="1.0",
                ASM_converter_name=ASM_CONVERTER_NAME,
                ASM_converter_version=ASM_CONVERTER_VERSION,
            ),
            calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                calculated_data_document=[
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="d006e7e7-fbe4-47cf-821b-904e85202803",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="d170f2df-7ff8-4978-b13d-a7ad19fab824",
                                    data_source_feature="normalized reporter result",
                                )
                            ]
                        ),
                        data_processing_document=None,
                        calculated_data_name="rn mean",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=1.261,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="f4fee39c-5861-4203-afcf-94ee755ac0b4",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="d170f2df-7ff8-4978-b13d-a7ad19fab824",
                                    data_source_feature="normalized reporter result",
                                )
                            ]
                        ),
                        data_processing_document=None,
                        calculated_data_name="rn sd",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.088,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="e6707b0c-4494-412f-8a8e-ef51d01f25b3",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="ba281c64-0605-4e76-8e9c-2a183be3cc08",
                                    data_source_feature="normalized reporter result",
                                )
                            ]
                        ),
                        data_processing_document=None,
                        calculated_data_name="rn mean",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.397,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="da78a225-2c34-40b3-b487-97893bfe491a",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="ba281c64-0605-4e76-8e9c-2a183be3cc08",
                                    data_source_feature="normalized reporter result",
                                )
                            ]
                        ),
                        data_processing_document=None,
                        calculated_data_name="rn sd",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.006,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                ]
            ),
        ),
        manifest="http://purl.allotrope.org/manifests/pcr/BENCHLING/2023/09/qpcr.manifest",
    )


def get_model2() -> Model:
    return Model(
        qPCR_aggregate_document=QPCRAggregateDocument(
            device_system_document=DeviceSystemDocument(
                device_identifier="278880086",
                model_number="ViiA 7",
                device_serial_number="278880086",
                asset_management_identifier=None,
                firmware_version=None,
                description=None,
                brand_name=None,
                product_manufacturer=None,
            ),
            qPCR_document=[
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=384,
                            unit="#",
                            field_type=None,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="1e572828d-45fd-49bd-ab66-2a7f06aea3b6",
                                measurement_time="2001-12-31T21:09:19-05:00",
                                target_DNA_description="B2M-Qiagen",
                                sample_document=SampleDocument(
                                    sample_identifier="1. 200217 U251p14_-ab_-SEMA3F_8h_pA_1",
                                    batch_identifier=None,
                                    sample_role_type="UNKNOWN",
                                    well_location_identifier="A1",
                                    well_plate_identifier=None,
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="SYBR_GREEN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="SYBR",
                                            quencher_dye_setting=None,
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.277,
                                                    unit="(unitless)",
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=None,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=18.717,
                                                unit="(unitless)",
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=None,
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.612]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[-0.007]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result=None,
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=ReporterDyeDataCube(
                                    label="reporter dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="reporter dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[34014.32]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                passive_reference_dye_data_cube=PassiveReferenceDyeDataCube(
                                    label="passive reference dye",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.integer,
                                                concept="cycle count",
                                                unit="#",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="passive reference dye fluorescence",
                                                unit="RFU",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[[55573.94]],  # type: ignore[list-item]
                                        points=None,
                                    ),
                                ),
                                melting_curve_data_cube=MeltingCurveDataCube(
                                    label="melting curve",
                                    cube_structure=TDatacubeStructure(
                                        dimensions=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="temperature",
                                                unit="degrees C",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            )
                                        ],
                                        measures=[
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="reporter dye fluorescence",
                                                unit="(unitless)",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            ),
                                            TDatacubeComponent(
                                                field_componentDatatype=FieldComponentDatatype.double,
                                                concept="slope",
                                                unit="(unitless)",
                                                scale=None,
                                                field_asm_fill_value=None,
                                            ),
                                        ],
                                    ),
                                    data=TDatacubeData(
                                        dimensions=[[1.0]],
                                        measures=[
                                            [3.478],  # type: ignore[list-item]
                                            [0.093],  # type: ignore[list-item]
                                        ],
                                        points=None,
                                    ),
                                ),
                            )
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="200224 U251p14 200217_14v9_SEMA3F_trial 1",
                        experiment_type=ExperimentType.comparative_CT_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                        well_volume=None,
                    ),
                    analyst=None,
                    submitter=None,
                )
            ],
            data_system_document=DataSystemDocument(
                data_system_instance_identifier="localhost",
                file_name="appbio_quantstudio_test02.txt",
                UNC_path="",
                software_name="Thermo QuantStudio",
                software_version="1.0",
                ASM_converter_name=ASM_CONVERTER_NAME,
                ASM_converter_version=ASM_CONVERTER_VERSION,
            ),
            calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                calculated_data_document=[],
            ),
        ),
        manifest="http://purl.allotrope.org/manifests/pcr/BENCHLING/2023/09/qpcr.manifest",
    )


def get_genotyping_data() -> Data:
    well = Well(
        identifier=0,
        items={
            "CYP19_2-Allele 1": WellItem(
                uuid="3c40d2c2-0a64-433f-b639-e9c6e896313e",
                identifier=1,
                position="A1",
                target_dna_description="CYP19_2-Allele 1",
                sample_identifier="NTC",
                well_location_identifier="A1",
                reporter_dye_setting=None,
                quencher_dye_setting=None,
                sample_role_type="NTC",
                amplification_data_obj=AmplificationData(
                    total_cycle_number_setting=2,
                    cycle=[1, 2],
                    rn=[0.275, 0.277],
                    delta_rn=[-0.003, -0.001],
                ),
                result_obj=Result(
                    cycle_threshold_value_setting=0.219,
                    cycle_threshold_result=None,
                    automatic_cycle_threshold_enabled_setting=True,
                    automatic_baseline_determination_enabled_setting=True,
                    normalized_reporter_result=None,
                    baseline_corrected_reporter_result=0.016,
                    genotyping_determination_result="Negative Control (NC)",
                    genotyping_determination_method_setting=None,
                    quantity=None,
                    quantity_mean=None,
                    quantity_sd=None,
                    ct_mean=None,
                    ct_sd=None,
                    delta_ct_mean=None,
                    delta_ct_se=None,
                    delta_delta_ct=None,
                    rq=None,
                    rq_min=None,
                    rq_max=None,
                    rn_mean=None,
                    rn_sd=None,
                    y_intercept=None,
                    r_squared=None,
                    slope=None,
                    efficiency=None,
                ),
            ),
            "CYP19_2-Allele 2": WellItem(
                uuid="cba0ba82-1486-4e0c-90ec-2abdfaf254b0",
                identifier=1,
                position="A1",
                target_dna_description="CYP19_2-Allele 2",
                sample_identifier="NTC",
                well_location_identifier="A1",
                reporter_dye_setting=None,
                quencher_dye_setting=None,
                sample_role_type="NTC",
                amplification_data_obj=AmplificationData(
                    total_cycle_number_setting=2,
                    cycle=[1, 2],
                    rn=[0.825, 0.831],
                    delta_rn=[-0.016, -0.011],
                ),
                result_obj=Result(
                    cycle_threshold_value_setting=0.132,
                    cycle_threshold_result=None,
                    automatic_cycle_threshold_enabled_setting=True,
                    automatic_baseline_determination_enabled_setting=True,
                    normalized_reporter_result=None,
                    baseline_corrected_reporter_result=0.029,
                    genotyping_determination_result="Negative Control (NC)",
                    genotyping_determination_method_setting=None,
                    quantity=None,
                    quantity_mean=None,
                    quantity_sd=None,
                    ct_mean=None,
                    ct_sd=None,
                    delta_ct_mean=None,
                    delta_ct_se=None,
                    delta_delta_ct=None,
                    rq=None,
                    rq_min=None,
                    rq_max=None,
                    rn_mean=None,
                    rn_sd=None,
                    y_intercept=None,
                    r_squared=None,
                    slope=None,
                    efficiency=None,
                ),
            ),
        },
    )

    return Data(
        header=Header(
            measurement_time="2010-09-16 07:35:29 AM EDT",
            plate_well_count=96,
            barcode=None,
            device_identifier="Sponge_Bob_32",
            model_number="QuantStudio(TM) 7 Flex System",
            device_serial_number="123456789",
            analyst=None,
            experimental_data_identifier="QuantStudio 96-Well SNP Genotyping Example",
            experiment_type=ExperimentType.genotyping_qPCR_experiment,
            measurement_method_identifier="Ct",
            pcr_detection_chemistry="TAQMAN",
            passive_reference_dye_setting="ROX",
        ),
        wells=WellList([well]),
        raw_data=None,
        endogenous_control="",
        reference_sample="",
        calculated_documents=[],
    )


def get_genotyping_model() -> Model:
    return Model(
        qPCR_aggregate_document=QPCRAggregateDocument(
            device_system_document=DeviceSystemDocument(
                device_identifier="Sponge_Bob_32",
                model_number="QuantStudio(TM) 7 Flex System",
                device_serial_number="123456789",
            ),
            qPCR_document=[
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=96,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="3c40d2c2-0a64-433f-b639-e9c6e896313e",
                                measurement_time="2010-09-16T07:35:29-04:00",
                                target_DNA_description="CYP19_2-Allele 1",
                                sample_document=SampleDocument(
                                    sample_identifier="NTC",
                                    sample_role_type="NTC",
                                    well_location_identifier="A1",
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=2.0
                                            ),
                                            reporter_dye_setting=None,
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.219
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=None
                                            ),
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0, 2.0]],
                                                    measures=[[0.275, 0.277]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=TQuantityValueUnitless(
                                                value=0.016
                                            ),
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0, 2.0]],
                                                    measures=[[-0.003, -0.001]],  # type: ignore[list-item]
                                                ),
                                            ),
                                            genotyping_determination_result="Negative Control (NC)",
                                        )
                                    ]
                                ),
                            ),
                            MeasurementDocumentItem(
                                measurement_identifier="cba0ba82-1486-4e0c-90ec-2abdfaf254b0",
                                measurement_time="2010-09-16T07:35:29-04:00",
                                target_DNA_description="CYP19_2-Allele 2",
                                sample_document=SampleDocument(
                                    sample_identifier="NTC",
                                    sample_role_type="NTC",
                                    well_location_identifier="A1",
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=2.0
                                            ),
                                            reporter_dye_setting=None,
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.132
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=None
                                            ),
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0, 2.0]],
                                                    measures=[[0.825, 0.831]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=TQuantityValueUnitless(
                                                value=0.029
                                            ),
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0, 2.0]],
                                                    measures=[[-0.016, -0.011]],  # type: ignore[list-item]
                                                ),
                                            ),
                                            genotyping_determination_result="Negative Control (NC)",
                                        )
                                    ]
                                ),
                            ),
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="QuantStudio 96-Well SNP Genotyping Example",
                        experiment_type=ExperimentType.genotyping_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                    ),
                ),
            ],
            data_system_document=DataSystemDocument(
                data_system_instance_identifier="localhost",
                file_name="appbio_quantstudio_test03.txt",
                UNC_path="",
                software_name="Thermo QuantStudio",
                software_version="1.0",
                ASM_converter_name=ASM_CONVERTER_NAME,
                ASM_converter_version=ASM_CONVERTER_VERSION,
            ),
            calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                calculated_data_document=[],
            ),
        ),
        manifest="http://purl.allotrope.org/manifests/pcr/BENCHLING/2023/09/qpcr.manifest",
    )


def get_rel_std_curve_data() -> Data:
    return Data(
        header=Header(
            measurement_time="2010-10-20 02:23:34 AM EDT",
            plate_well_count=96,
            experiment_type=ExperimentType.relative_standard_curve_qPCR_experiment,
            device_identifier="278880034",
            model_number="QuantStudio(TM) 7 Flex System",
            device_serial_number="278880034",
            measurement_method_identifier="Ct",
            pcr_detection_chemistry="TAQMAN",
            passive_reference_dye_setting="ROX",
            barcode="",
            analyst=None,
            experimental_data_identifier="QuantStudio96-Well Relative Standard Curve Example",
        ),
        wells=WellList(
            wells=[
                Well(
                    identifier=37,
                    items={
                        "RNaseP": WellItem(
                            uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                            identifier=37,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D1",
                            well_location_identifier="D1",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.627],
                                delta_rn=[0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.155,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=794.91,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        )
                    },
                    multicomponent_data=None,
                    melt_curve_raw_data=None,
                    calculated_document=CalculatedDocument(
                        uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                        name="quantity",
                        value=794.91,
                        data_sources=[
                            DataSource(
                                feature="cycle threshold result",
                                reference=WellItem(
                                    uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                    identifier=37,
                                    target_dna_description="RNaseP",
                                    sample_identifier="800",
                                    reporter_dye_setting="FAM",
                                    position="D1",
                                    well_location_identifier="D1",
                                    quencher_dye_setting="NFQ-MGB",
                                    sample_role_type="UNKNOWN",
                                    amplification_data_obj=AmplificationData(
                                        total_cycle_number_setting=1.0,
                                        cycle=[1],
                                        rn=[0.627],
                                        delta_rn=[0.001],
                                    ),
                                    result_obj=Result(
                                        cycle_threshold_value_setting=0.133,
                                        cycle_threshold_result=30.155,
                                        automatic_cycle_threshold_enabled_setting=True,
                                        automatic_baseline_determination_enabled_setting=True,
                                        normalized_reporter_result=None,
                                        baseline_corrected_reporter_result=None,
                                        genotyping_determination_result=None,
                                        genotyping_determination_method_setting=None,
                                        quantity=794.91,
                                        quantity_mean=818.012,
                                        quantity_sd=29.535,
                                        ct_mean=30.115,
                                        ct_sd=0.051,
                                        delta_ct_mean=None,
                                        delta_ct_se=None,
                                        delta_delta_ct=None,
                                        rq=0.798,
                                        rq_min=0.658,
                                        rq_max=0.967,
                                        rn_mean=None,
                                        rn_sd=None,
                                        y_intercept=39.662,
                                        r_squared=0.999,
                                        slope=-3.278,
                                        efficiency=101.866,
                                    ),
                                ),
                            )
                        ],
                        iterated=True,
                    ),
                ),
                Well(
                    identifier=38,
                    items={
                        "RNaseP": WellItem(
                            uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                            identifier=38,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D2",
                            well_location_identifier="D2",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.612],
                                delta_rn=[-0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.2,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=769.776,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        )
                    },
                    multicomponent_data=None,
                    melt_curve_raw_data=None,
                    calculated_document=CalculatedDocument(
                        uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                        name="quantity",
                        value=769.776,
                        data_sources=[
                            DataSource(
                                feature="cycle threshold result",
                                reference=WellItem(
                                    uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                    identifier=38,
                                    target_dna_description="RNaseP",
                                    sample_identifier="800",
                                    reporter_dye_setting="FAM",
                                    position="D2",
                                    well_location_identifier="D2",
                                    quencher_dye_setting="NFQ-MGB",
                                    sample_role_type="UNKNOWN",
                                    amplification_data_obj=AmplificationData(
                                        total_cycle_number_setting=1.0,
                                        cycle=[1],
                                        rn=[0.612],
                                        delta_rn=[-0.001],
                                    ),
                                    result_obj=Result(
                                        cycle_threshold_value_setting=0.133,
                                        cycle_threshold_result=30.2,
                                        automatic_cycle_threshold_enabled_setting=True,
                                        automatic_baseline_determination_enabled_setting=True,
                                        normalized_reporter_result=None,
                                        baseline_corrected_reporter_result=None,
                                        genotyping_determination_result=None,
                                        genotyping_determination_method_setting=None,
                                        quantity=769.776,
                                        quantity_mean=818.012,
                                        quantity_sd=29.535,
                                        ct_mean=30.115,
                                        ct_sd=0.051,
                                        delta_ct_mean=None,
                                        delta_ct_se=None,
                                        delta_delta_ct=None,
                                        rq=0.798,
                                        rq_min=0.658,
                                        rq_max=0.967,
                                        rn_mean=None,
                                        rn_sd=None,
                                        y_intercept=39.662,
                                        r_squared=0.999,
                                        slope=-3.278,
                                        efficiency=101.866,
                                    ),
                                ),
                            )
                        ],
                        iterated=True,
                    ),
                ),
            ]
        ),
        raw_data=None,
        endogenous_control="RNaseP",
        reference_sample="800",
        calculated_documents=[
            CalculatedDocument(
                uuid="9b24577d-08bf-4312-8d1e-13feea315b0f",
                name="quantity mean",
                value=818.012,
                data_sources=[
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                            name="quantity",
                            value=794.91,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                        identifier=37,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D1",
                                        well_location_identifier="D1",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.627],
                                            delta_rn=[0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.155,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=794.91,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                            name="quantity",
                            value=769.776,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                        identifier=38,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D2",
                                        well_location_identifier="D2",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.612],
                                            delta_rn=[-0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.2,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=769.776,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="7434c27a-11d2-41d4-b20d-7407b8dec622",
                name="quantity sd",
                value=29.535,
                data_sources=[
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                            name="quantity",
                            value=794.91,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                        identifier=37,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D1",
                                        well_location_identifier="D1",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.627],
                                            delta_rn=[0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.155,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=794.91,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                            name="quantity",
                            value=769.776,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                        identifier=38,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D2",
                                        well_location_identifier="D2",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.612],
                                            delta_rn=[-0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.2,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=769.776,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="88749c03-5cdb-49c1-b829-092e95e863ff",
                name="ct sd",
                value=0.051,
                data_sources=[
                    DataSource(
                        feature="cycle threshold result",
                        reference=WellItem(
                            uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                            identifier=37,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D1",
                            well_location_identifier="D1",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.627],
                                delta_rn=[0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.155,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=794.91,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        ),
                    ),
                    DataSource(
                        feature="cycle threshold result",
                        reference=WellItem(
                            uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                            identifier=38,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D2",
                            well_location_identifier="D2",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.612],
                                delta_rn=[-0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.2,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=769.776,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        ),
                    ),
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="75798875-017f-4026-9131-5d7661e01213",
                name="rq min",
                value=0.658,
                data_sources=[
                    DataSource(
                        feature="rq",
                        reference=CalculatedDocument(
                            uuid="4606bf27-7d50-4ef0-a643-6d105b080057",
                            name="rq",
                            value=0.798,
                            data_sources=[
                                DataSource(
                                    feature="quantity mean",
                                    reference=CalculatedDocument(
                                        uuid="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                                        name="quantity mean",
                                        value=818.012,
                                        data_sources=[
                                            DataSource(
                                                feature="quantity",
                                                reference=CalculatedDocument(
                                                    uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                                                    name="quantity",
                                                    value=794.91,
                                                    data_sources=[
                                                        DataSource(
                                                            feature="cycle threshold result",
                                                            reference=WellItem(
                                                                uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                                                identifier=37,
                                                                target_dna_description="RNaseP",
                                                                sample_identifier="800",
                                                                reporter_dye_setting="FAM",
                                                                position="D1",
                                                                well_location_identifier="D1",
                                                                quencher_dye_setting="NFQ-MGB",
                                                                sample_role_type="UNKNOWN",
                                                                amplification_data_obj=AmplificationData(
                                                                    total_cycle_number_setting=1.0,
                                                                    cycle=[1],
                                                                    rn=[0.627],
                                                                    delta_rn=[0.001],
                                                                ),
                                                                result_obj=Result(
                                                                    cycle_threshold_value_setting=0.133,
                                                                    cycle_threshold_result=30.155,
                                                                    automatic_cycle_threshold_enabled_setting=True,
                                                                    automatic_baseline_determination_enabled_setting=True,
                                                                    normalized_reporter_result=None,
                                                                    baseline_corrected_reporter_result=None,
                                                                    genotyping_determination_result=None,
                                                                    genotyping_determination_method_setting=None,
                                                                    quantity=794.91,
                                                                    quantity_mean=818.012,
                                                                    quantity_sd=29.535,
                                                                    ct_mean=30.115,
                                                                    ct_sd=0.051,
                                                                    delta_ct_mean=None,
                                                                    delta_ct_se=None,
                                                                    delta_delta_ct=None,
                                                                    rq=0.798,
                                                                    rq_min=0.658,
                                                                    rq_max=0.967,
                                                                    rn_mean=None,
                                                                    rn_sd=None,
                                                                    y_intercept=39.662,
                                                                    r_squared=0.999,
                                                                    slope=-3.278,
                                                                    efficiency=101.866,
                                                                ),
                                                            ),
                                                        )
                                                    ],
                                                    iterated=True,
                                                ),
                                            ),
                                            DataSource(
                                                feature="quantity",
                                                reference=CalculatedDocument(
                                                    uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                                    name="quantity",
                                                    value=769.776,
                                                    data_sources=[
                                                        DataSource(
                                                            feature="cycle threshold result",
                                                            reference=WellItem(
                                                                uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                                                identifier=38,
                                                                target_dna_description="RNaseP",
                                                                sample_identifier="800",
                                                                reporter_dye_setting="FAM",
                                                                position="D2",
                                                                well_location_identifier="D2",
                                                                quencher_dye_setting="NFQ-MGB",
                                                                sample_role_type="UNKNOWN",
                                                                amplification_data_obj=AmplificationData(
                                                                    total_cycle_number_setting=1.0,
                                                                    cycle=[1],
                                                                    rn=[0.612],
                                                                    delta_rn=[-0.001],
                                                                ),
                                                                result_obj=Result(
                                                                    cycle_threshold_value_setting=0.133,
                                                                    cycle_threshold_result=30.2,
                                                                    automatic_cycle_threshold_enabled_setting=True,
                                                                    automatic_baseline_determination_enabled_setting=True,
                                                                    normalized_reporter_result=None,
                                                                    baseline_corrected_reporter_result=None,
                                                                    genotyping_determination_result=None,
                                                                    genotyping_determination_method_setting=None,
                                                                    quantity=769.776,
                                                                    quantity_mean=818.012,
                                                                    quantity_sd=29.535,
                                                                    ct_mean=30.115,
                                                                    ct_sd=0.051,
                                                                    delta_ct_mean=None,
                                                                    delta_ct_se=None,
                                                                    delta_delta_ct=None,
                                                                    rq=0.798,
                                                                    rq_min=0.658,
                                                                    rq_max=0.967,
                                                                    rn_mean=None,
                                                                    rn_sd=None,
                                                                    y_intercept=39.662,
                                                                    r_squared=0.999,
                                                                    slope=-3.278,
                                                                    efficiency=101.866,
                                                                ),
                                                            ),
                                                        )
                                                    ],
                                                    iterated=True,
                                                ),
                                            ),
                                        ],
                                        iterated=True,
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    )
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="4606bf27-7d50-4ef0-a643-6d105b080057",
                name="rq",
                value=0.798,
                data_sources=[
                    DataSource(
                        feature="quantity mean",
                        reference=CalculatedDocument(
                            uuid="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                            name="quantity mean",
                            value=818.012,
                            data_sources=[
                                DataSource(
                                    feature="quantity",
                                    reference=CalculatedDocument(
                                        uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                                        name="quantity",
                                        value=794.91,
                                        data_sources=[
                                            DataSource(
                                                feature="cycle threshold result",
                                                reference=WellItem(
                                                    uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                                    identifier=37,
                                                    target_dna_description="RNaseP",
                                                    sample_identifier="800",
                                                    reporter_dye_setting="FAM",
                                                    position="D1",
                                                    well_location_identifier="D1",
                                                    quencher_dye_setting="NFQ-MGB",
                                                    sample_role_type="UNKNOWN",
                                                    amplification_data_obj=AmplificationData(
                                                        total_cycle_number_setting=1.0,
                                                        cycle=[1],
                                                        rn=[0.627],
                                                        delta_rn=[0.001],
                                                    ),
                                                    result_obj=Result(
                                                        cycle_threshold_value_setting=0.133,
                                                        cycle_threshold_result=30.155,
                                                        automatic_cycle_threshold_enabled_setting=True,
                                                        automatic_baseline_determination_enabled_setting=True,
                                                        normalized_reporter_result=None,
                                                        baseline_corrected_reporter_result=None,
                                                        genotyping_determination_result=None,
                                                        genotyping_determination_method_setting=None,
                                                        quantity=794.91,
                                                        quantity_mean=818.012,
                                                        quantity_sd=29.535,
                                                        ct_mean=30.115,
                                                        ct_sd=0.051,
                                                        delta_ct_mean=None,
                                                        delta_ct_se=None,
                                                        delta_delta_ct=None,
                                                        rq=0.798,
                                                        rq_min=0.658,
                                                        rq_max=0.967,
                                                        rn_mean=None,
                                                        rn_sd=None,
                                                        y_intercept=39.662,
                                                        r_squared=0.999,
                                                        slope=-3.278,
                                                        efficiency=101.866,
                                                    ),
                                                ),
                                            )
                                        ],
                                        iterated=True,
                                    ),
                                ),
                                DataSource(
                                    feature="quantity",
                                    reference=CalculatedDocument(
                                        uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                        name="quantity",
                                        value=769.776,
                                        data_sources=[
                                            DataSource(
                                                feature="cycle threshold result",
                                                reference=WellItem(
                                                    uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                                    identifier=38,
                                                    target_dna_description="RNaseP",
                                                    sample_identifier="800",
                                                    reporter_dye_setting="FAM",
                                                    position="D2",
                                                    well_location_identifier="D2",
                                                    quencher_dye_setting="NFQ-MGB",
                                                    sample_role_type="UNKNOWN",
                                                    amplification_data_obj=AmplificationData(
                                                        total_cycle_number_setting=1.0,
                                                        cycle=[1],
                                                        rn=[0.612],
                                                        delta_rn=[-0.001],
                                                    ),
                                                    result_obj=Result(
                                                        cycle_threshold_value_setting=0.133,
                                                        cycle_threshold_result=30.2,
                                                        automatic_cycle_threshold_enabled_setting=True,
                                                        automatic_baseline_determination_enabled_setting=True,
                                                        normalized_reporter_result=None,
                                                        baseline_corrected_reporter_result=None,
                                                        genotyping_determination_result=None,
                                                        genotyping_determination_method_setting=None,
                                                        quantity=769.776,
                                                        quantity_mean=818.012,
                                                        quantity_sd=29.535,
                                                        ct_mean=30.115,
                                                        ct_sd=0.051,
                                                        delta_ct_mean=None,
                                                        delta_ct_se=None,
                                                        delta_delta_ct=None,
                                                        rq=0.798,
                                                        rq_min=0.658,
                                                        rq_max=0.967,
                                                        rn_mean=None,
                                                        rn_sd=None,
                                                        y_intercept=39.662,
                                                        r_squared=0.999,
                                                        slope=-3.278,
                                                        efficiency=101.866,
                                                    ),
                                                ),
                                            )
                                        ],
                                        iterated=True,
                                    ),
                                ),
                            ],
                            iterated=True,
                        ),
                    )
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                name="quantity mean",
                value=818.012,
                data_sources=[
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                            name="quantity",
                            value=794.91,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                        identifier=37,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D1",
                                        well_location_identifier="D1",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.627],
                                            delta_rn=[0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.155,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=794.91,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                    DataSource(
                        feature="quantity",
                        reference=CalculatedDocument(
                            uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                            name="quantity",
                            value=769.776,
                            data_sources=[
                                DataSource(
                                    feature="cycle threshold result",
                                    reference=WellItem(
                                        uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                        identifier=38,
                                        target_dna_description="RNaseP",
                                        sample_identifier="800",
                                        reporter_dye_setting="FAM",
                                        position="D2",
                                        well_location_identifier="D2",
                                        quencher_dye_setting="NFQ-MGB",
                                        sample_role_type="UNKNOWN",
                                        amplification_data_obj=AmplificationData(
                                            total_cycle_number_setting=1.0,
                                            cycle=[1],
                                            rn=[0.612],
                                            delta_rn=[-0.001],
                                        ),
                                        result_obj=Result(
                                            cycle_threshold_value_setting=0.133,
                                            cycle_threshold_result=30.2,
                                            automatic_cycle_threshold_enabled_setting=True,
                                            automatic_baseline_determination_enabled_setting=True,
                                            normalized_reporter_result=None,
                                            baseline_corrected_reporter_result=None,
                                            genotyping_determination_result=None,
                                            genotyping_determination_method_setting=None,
                                            quantity=769.776,
                                            quantity_mean=818.012,
                                            quantity_sd=29.535,
                                            ct_mean=30.115,
                                            ct_sd=0.051,
                                            delta_ct_mean=None,
                                            delta_ct_se=None,
                                            delta_delta_ct=None,
                                            rq=0.798,
                                            rq_min=0.658,
                                            rq_max=0.967,
                                            rn_mean=None,
                                            rn_sd=None,
                                            y_intercept=39.662,
                                            r_squared=0.999,
                                            slope=-3.278,
                                            efficiency=101.866,
                                        ),
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    ),
                ],
                iterated=True,
            ),
            CalculatedDocument(
                uuid="d23ae1c4-e750-43eb-a6df-ed66d96bcd20",
                name="rq max",
                value=0.967,
                data_sources=[
                    DataSource(
                        feature="rq",
                        reference=CalculatedDocument(
                            uuid="4606bf27-7d50-4ef0-a643-6d105b080057",
                            name="rq",
                            value=0.798,
                            data_sources=[
                                DataSource(
                                    feature="quantity mean",
                                    reference=CalculatedDocument(
                                        uuid="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                                        name="quantity mean",
                                        value=818.012,
                                        data_sources=[
                                            DataSource(
                                                feature="quantity",
                                                reference=CalculatedDocument(
                                                    uuid="94cf744d-d243-47bd-a723-aca6c6517af5",
                                                    name="quantity",
                                                    value=794.91,
                                                    data_sources=[
                                                        DataSource(
                                                            feature="cycle threshold result",
                                                            reference=WellItem(
                                                                uuid="2b290825-2a40-4768-a21f-a8a603c32c85",
                                                                identifier=37,
                                                                target_dna_description="RNaseP",
                                                                sample_identifier="800",
                                                                reporter_dye_setting="FAM",
                                                                position="D1",
                                                                well_location_identifier="D1",
                                                                quencher_dye_setting="NFQ-MGB",
                                                                sample_role_type="UNKNOWN",
                                                                amplification_data_obj=AmplificationData(
                                                                    total_cycle_number_setting=1.0,
                                                                    cycle=[1],
                                                                    rn=[0.627],
                                                                    delta_rn=[0.001],
                                                                ),
                                                                result_obj=Result(
                                                                    cycle_threshold_value_setting=0.133,
                                                                    cycle_threshold_result=30.155,
                                                                    automatic_cycle_threshold_enabled_setting=True,
                                                                    automatic_baseline_determination_enabled_setting=True,
                                                                    normalized_reporter_result=None,
                                                                    baseline_corrected_reporter_result=None,
                                                                    genotyping_determination_result=None,
                                                                    genotyping_determination_method_setting=None,
                                                                    quantity=794.91,
                                                                    quantity_mean=818.012,
                                                                    quantity_sd=29.535,
                                                                    ct_mean=30.115,
                                                                    ct_sd=0.051,
                                                                    delta_ct_mean=None,
                                                                    delta_ct_se=None,
                                                                    delta_delta_ct=None,
                                                                    rq=0.798,
                                                                    rq_min=0.658,
                                                                    rq_max=0.967,
                                                                    rn_mean=None,
                                                                    rn_sd=None,
                                                                    y_intercept=39.662,
                                                                    r_squared=0.999,
                                                                    slope=-3.278,
                                                                    efficiency=101.866,
                                                                ),
                                                            ),
                                                        )
                                                    ],
                                                    iterated=True,
                                                ),
                                            ),
                                            DataSource(
                                                feature="quantity",
                                                reference=CalculatedDocument(
                                                    uuid="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                                    name="quantity",
                                                    value=769.776,
                                                    data_sources=[
                                                        DataSource(
                                                            feature="cycle threshold result",
                                                            reference=WellItem(
                                                                uuid="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                                                identifier=38,
                                                                target_dna_description="RNaseP",
                                                                sample_identifier="800",
                                                                reporter_dye_setting="FAM",
                                                                position="D2",
                                                                well_location_identifier="D2",
                                                                quencher_dye_setting="NFQ-MGB",
                                                                sample_role_type="UNKNOWN",
                                                                amplification_data_obj=AmplificationData(
                                                                    total_cycle_number_setting=1.0,
                                                                    cycle=[1],
                                                                    rn=[0.612],
                                                                    delta_rn=[-0.001],
                                                                ),
                                                                result_obj=Result(
                                                                    cycle_threshold_value_setting=0.133,
                                                                    cycle_threshold_result=30.2,
                                                                    automatic_cycle_threshold_enabled_setting=True,
                                                                    automatic_baseline_determination_enabled_setting=True,
                                                                    normalized_reporter_result=None,
                                                                    baseline_corrected_reporter_result=None,
                                                                    genotyping_determination_result=None,
                                                                    genotyping_determination_method_setting=None,
                                                                    quantity=769.776,
                                                                    quantity_mean=818.012,
                                                                    quantity_sd=29.535,
                                                                    ct_mean=30.115,
                                                                    ct_sd=0.051,
                                                                    delta_ct_mean=None,
                                                                    delta_ct_se=None,
                                                                    delta_delta_ct=None,
                                                                    rq=0.798,
                                                                    rq_min=0.658,
                                                                    rq_max=0.967,
                                                                    rn_mean=None,
                                                                    rn_sd=None,
                                                                    y_intercept=39.662,
                                                                    r_squared=0.999,
                                                                    slope=-3.278,
                                                                    efficiency=101.866,
                                                                ),
                                                            ),
                                                        )
                                                    ],
                                                    iterated=True,
                                                ),
                                            ),
                                        ],
                                        iterated=True,
                                    ),
                                )
                            ],
                            iterated=True,
                        ),
                    )
                ],
                iterated=True,
            ),
        ],
    )


def get_rel_std_curve_model() -> Model:
    return Model(
        qPCR_aggregate_document=QPCRAggregateDocument(
            device_system_document=DeviceSystemDocument(
                device_identifier="278880034",
                model_number="QuantStudio(TM) 7 Flex System",
                device_serial_number="278880034",
                asset_management_identifier=None,
                firmware_version=None,
                description=None,
                brand_name=None,
                product_manufacturer=None,
            ),
            qPCR_document=[
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=96,
                            unit="#",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="2b290825-2a40-4768-a21f-a8a603c32c85",
                                measurement_time="2010-10-20T02:23:34-04:00",
                                target_DNA_description="RNaseP",
                                sample_document=SampleDocument(
                                    sample_identifier="800",
                                    batch_identifier=None,
                                    sample_role_type="UNKNOWN",
                                    well_location_identifier="D1",
                                    well_plate_identifier="",
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="FAM",
                                            quencher_dye_setting="NFQ-MGB",
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.133,
                                                    unit="(unitless)",
                                                    has_statistic_datum_role=None,
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=None,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=30.155,
                                                unit="(unitless)",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=None,
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.627]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.001]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result=None,
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=None,
                                passive_reference_dye_data_cube=None,
                                melting_curve_data_cube=None,
                            )
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="QuantStudio96-Well Relative Standard Curve Example",
                        experiment_type=ExperimentType.relative_standard_curve_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                        well_volume=None,
                    ),
                    analyst=None,
                    submitter=None,
                    calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                        calculated_data_document=[
                            CalculatedDataDocumentItem(
                                calculated_data_identifier="94cf744d-d243-47bd-a723-aca6c6517af5",
                                data_source_aggregate_document=DataSourceAggregateDocument(
                                    data_source_document=[
                                        DataSourceDocumentItem(
                                            data_source_identifier="2b290825-2a40-4768-a21f-a8a603c32c85",
                                            data_source_feature="cycle threshold result",
                                        )
                                    ]
                                ),
                                data_processing_document=None,
                                calculated_data_name="quantity",
                                calculated_data_description=None,
                                calculated_datum=TQuantityValueUnitless(
                                    value=794.91,
                                    unit="(unitless)",
                                    has_statistic_datum_role=None,
                                    field_type=None,
                                ),
                            )
                        ]
                    ),
                ),
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=96,
                            unit="#",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                measurement_time="2010-10-20T02:23:34-04:00",
                                target_DNA_description="RNaseP",
                                sample_document=SampleDocument(
                                    sample_identifier="800",
                                    batch_identifier=None,
                                    sample_role_type="UNKNOWN",
                                    well_location_identifier="D2",
                                    well_plate_identifier="",
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="FAM",
                                            quencher_dye_setting="NFQ-MGB",
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.133,
                                                    unit="(unitless)",
                                                    has_statistic_datum_role=None,
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=None,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=30.2,
                                                unit="(unitless)",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=None,
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.612]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[-0.001]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result=None,
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=None,
                                passive_reference_dye_data_cube=None,
                                melting_curve_data_cube=None,
                            )
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="QuantStudio96-Well Relative Standard Curve Example",
                        experiment_type=ExperimentType.relative_standard_curve_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                        well_volume=None,
                    ),
                    analyst=None,
                    submitter=None,
                    calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                        calculated_data_document=[
                            CalculatedDataDocumentItem(
                                calculated_data_identifier="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                data_source_aggregate_document=DataSourceAggregateDocument(
                                    data_source_document=[
                                        DataSourceDocumentItem(
                                            data_source_identifier="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                            data_source_feature="cycle threshold result",
                                        )
                                    ]
                                ),
                                data_processing_document=None,
                                calculated_data_name="quantity",
                                calculated_data_description=None,
                                calculated_datum=TQuantityValueUnitless(
                                    value=769.776,
                                    unit="(unitless)",
                                    has_statistic_datum_role=None,
                                    field_type=None,
                                ),
                            )
                        ]
                    ),
                ),
            ],
            data_system_document=DataSystemDocument(
                data_system_instance_identifier="localhost",
                file_name="appbio_quantstudio_test04.txt",
                UNC_path="",
                software_name="Thermo QuantStudio",
                software_version="1.0",
                ASM_converter_name="allotropy",
                ASM_converter_version=ASM_CONVERTER_VERSION,
            ),
            calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                calculated_data_document=[
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="9b24577d-08bf-4312-8d1e-13feea315b0f",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="94cf744d-d243-47bd-a723-aca6c6517af5",
                                    data_source_feature="quantity",
                                ),
                                DataSourceDocumentItem(
                                    data_source_identifier="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                    data_source_feature="quantity",
                                ),
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="quantity mean",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=818.012,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="7434c27a-11d2-41d4-b20d-7407b8dec622",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="94cf744d-d243-47bd-a723-aca6c6517af5",
                                    data_source_feature="quantity",
                                ),
                                DataSourceDocumentItem(
                                    data_source_identifier="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                    data_source_feature="quantity",
                                ),
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="quantity sd",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=29.535,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="88749c03-5cdb-49c1-b829-092e95e863ff",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="2b290825-2a40-4768-a21f-a8a603c32c85",
                                    data_source_feature="cycle threshold result",
                                ),
                                DataSourceDocumentItem(
                                    data_source_identifier="42788e09-e972-4c9b-b8d0-7263e07371fa",
                                    data_source_feature="cycle threshold result",
                                ),
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="ct sd",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.051,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="75798875-017f-4026-9131-5d7661e01213",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="4606bf27-7d50-4ef0-a643-6d105b080057",
                                    data_source_feature="rq",
                                )
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="rq min",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.658,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="4606bf27-7d50-4ef0-a643-6d105b080057",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                                    data_source_feature="quantity mean",
                                )
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="rq",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.798,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="0eb90790-c6a6-476e-b5cb-8b0c6a3e62f2",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="94cf744d-d243-47bd-a723-aca6c6517af5",
                                    data_source_feature="quantity",
                                ),
                                DataSourceDocumentItem(
                                    data_source_identifier="51066bef-10b0-4782-a8c9-ee7383fe3256",
                                    data_source_feature="quantity",
                                ),
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="quantity mean",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=818.012,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="d23ae1c4-e750-43eb-a6df-ed66d96bcd20",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="4606bf27-7d50-4ef0-a643-6d105b080057",
                                    data_source_feature="rq",
                                )
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="rq max",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.967,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    ),
                ]
            ),
        ),
        manifest="http://purl.allotrope.org/manifests/pcr/BENCHLING/2023/09/qpcr.manifest",
    )


def get_broken_calc_doc_data() -> Data:
    return Data(
        header=Header(
            measurement_time="2010-10-20 02:23:34 AM EDT",
            plate_well_count=96,
            experiment_type=ExperimentType.relative_standard_curve_qPCR_experiment,
            device_identifier="278880034",
            model_number="QuantStudio(TM) 7 Flex System",
            device_serial_number="278880034",
            measurement_method_identifier="Ct",
            pcr_detection_chemistry="TAQMAN",
            passive_reference_dye_setting="ROX",
            barcode="",
            analyst=None,
            experimental_data_identifier="QuantStudio96-Well Relative Standard Curve Example",
        ),
        wells=WellList(
            wells=[
                Well(
                    identifier=37,
                    items={
                        "RNaseP": WellItem(
                            uuid="a1453d7a-dd6b-4630-923f-7e4a14a7ee70",
                            identifier=37,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D1",
                            well_location_identifier="D1",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.627],
                                delta_rn=[0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.155,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=None,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        )
                    },
                    multicomponent_data=None,
                    melt_curve_raw_data=None,
                    calculated_document=None,
                )
            ]
        ),
        raw_data=None,
        endogenous_control="RNaseP",
        reference_sample="800",
        calculated_documents=[
            CalculatedDocument(
                uuid="d6d4d929-aa30-414b-a56a-12d2cac78bb3",
                name="ct sd",
                value=0.051,
                data_sources=[
                    DataSource(
                        feature="cycle threshold result",
                        reference=WellItem(
                            uuid="a1453d7a-dd6b-4630-923f-7e4a14a7ee70",
                            identifier=37,
                            target_dna_description="RNaseP",
                            sample_identifier="800",
                            reporter_dye_setting="FAM",
                            position="D1",
                            well_location_identifier="D1",
                            quencher_dye_setting="NFQ-MGB",
                            sample_role_type="UNKNOWN",
                            amplification_data_obj=AmplificationData(
                                total_cycle_number_setting=1.0,
                                cycle=[1],
                                rn=[0.627],
                                delta_rn=[0.001],
                            ),
                            result_obj=Result(
                                cycle_threshold_value_setting=0.133,
                                cycle_threshold_result=30.155,
                                automatic_cycle_threshold_enabled_setting=True,
                                automatic_baseline_determination_enabled_setting=True,
                                normalized_reporter_result=None,
                                baseline_corrected_reporter_result=None,
                                genotyping_determination_result=None,
                                genotyping_determination_method_setting=None,
                                quantity=None,
                                quantity_mean=818.012,
                                quantity_sd=29.535,
                                ct_mean=30.115,
                                ct_sd=0.051,
                                delta_ct_mean=None,
                                delta_ct_se=None,
                                delta_delta_ct=None,
                                rq=0.798,
                                rq_min=0.658,
                                rq_max=0.967,
                                rn_mean=None,
                                rn_sd=None,
                                y_intercept=39.662,
                                r_squared=0.999,
                                slope=-3.278,
                                efficiency=101.866,
                            ),
                        ),
                    )
                ],
                iterated=True,
            )
        ],
    )


def get_broken_calc_doc_model() -> Model:
    return Model(
        qPCR_aggregate_document=QPCRAggregateDocument(
            device_system_document=DeviceSystemDocument(
                device_identifier="278880034",
                model_number="QuantStudio(TM) 7 Flex System",
                device_serial_number="278880034",
                asset_management_identifier=None,
                firmware_version=None,
                description=None,
                brand_name=None,
                product_manufacturer=None,
            ),
            qPCR_document=[
                QPCRDocumentItem(
                    measurement_aggregate_document=MeasurementAggregateDocument(
                        plate_well_count=TQuantityValueNumber(
                            value=96,
                            unit="#",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                        measurement_document=[
                            MeasurementDocumentItem(
                                measurement_identifier="a1453d7a-dd6b-4630-923f-7e4a14a7ee70",
                                measurement_time="2010-10-20T02:23:34-04:00",
                                target_DNA_description="RNaseP",
                                sample_document=SampleDocument(
                                    sample_identifier="800",
                                    batch_identifier=None,
                                    sample_role_type="UNKNOWN",
                                    well_location_identifier="D1",
                                    well_plate_identifier="",
                                    mass_concentration=None,
                                ),
                                device_control_aggregate_document=DeviceControlAggregateDocument(
                                    device_control_document=[
                                        DeviceControlDocumentItem(
                                            device_type="qPCR",
                                            measurement_method_identifier="Ct",
                                            PCR_detection_chemistry="TAQMAN",
                                            device_identifier=None,
                                            detection_type=None,
                                            total_cycle_number_setting=TQuantityValueNumber(
                                                value=1.0,
                                                unit="#",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            denaturing_temperature_setting=None,
                                            denaturing_time_setting=None,
                                            annealing_temperature_setting=None,
                                            annealing_time_setting=None,
                                            extension_temperature_setting=None,
                                            extension_time_setting=None,
                                            reporter_dye_setting="FAM",
                                            quencher_dye_setting="NFQ-MGB",
                                            passive_reference_dye_setting="ROX",
                                        )
                                    ]
                                ),
                                processed_data_aggregate_document=ProcessedDataAggregateDocument(
                                    processed_data_document=[
                                        ProcessedDataDocumentItem(
                                            data_processing_document=DataProcessingDocument(
                                                cycle_threshold_value_setting=TQuantityValueUnitless(
                                                    value=0.133,
                                                    unit="(unitless)",
                                                    has_statistic_datum_role=None,
                                                    field_type=None,
                                                ),
                                                automatic_cycle_threshold_enabled_setting=True,
                                                automatic_baseline_determination_enabled_setting=True,
                                                baseline_determination_start_cycle_setting=None,
                                                baseline_determination_end_cycle_setting=None,
                                                genotyping_determination_method=None,
                                                genotyping_determination_method_setting=None,
                                            ),
                                            cycle_threshold_result=TNullableQuantityValueUnitless(
                                                value=30.155,
                                                unit="(unitless)",
                                                has_statistic_datum_role=None,
                                                field_type=None,
                                            ),
                                            normalized_reporter_result=None,
                                            normalized_reporter_data_cube=NormalizedReporterDataCube(
                                                label="normalized reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="normalized report result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.627]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            baseline_corrected_reporter_result=None,
                                            baseline_corrected_reporter_data_cube=BaselineCorrectedReporterDataCube(
                                                label="baseline corrected reporter",
                                                cube_structure=TDatacubeStructure(
                                                    dimensions=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.integer,
                                                            concept="cycle count",
                                                            unit="#",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                    measures=[
                                                        TDatacubeComponent(
                                                            field_componentDatatype=FieldComponentDatatype.double,
                                                            concept="baseline corrected reporter result",
                                                            unit="(unitless)",
                                                            scale=None,
                                                            field_asm_fill_value=None,
                                                        )
                                                    ],
                                                ),
                                                data=TDatacubeData(
                                                    dimensions=[[1.0]],
                                                    measures=[[0.001]],  # type: ignore[list-item]
                                                    points=None,
                                                ),
                                            ),
                                            genotyping_determination_result=None,
                                        )
                                    ]
                                ),
                                reporter_dye_data_cube=None,
                                passive_reference_dye_data_cube=None,
                                melting_curve_data_cube=None,
                            )
                        ],
                        analytical_method_identifier=None,
                        experimental_data_identifier="QuantStudio96-Well Relative Standard Curve Example",
                        experiment_type=ExperimentType.relative_standard_curve_qPCR_experiment,
                        container_type=ContainerType.qPCR_reaction_block,
                        well_volume=None,
                    ),
                    analyst=None,
                    submitter=None,
                    calculated_data_aggregate_document=None,
                )
            ],
            data_system_document=DataSystemDocument(
                data_system_instance_identifier="localhost",
                file_name="appbio_quantstudio_test05.txt",
                UNC_path="",
                software_name="Thermo QuantStudio",
                software_version="1.0",
                ASM_converter_name="allotropy",
                ASM_converter_version=ASM_CONVERTER_VERSION,
            ),
            calculated_data_aggregate_document=TCalculatedDataAggregateDocument(
                calculated_data_document=[
                    CalculatedDataDocumentItem(
                        calculated_data_identifier="d6d4d929-aa30-414b-a56a-12d2cac78bb3",
                        data_source_aggregate_document=DataSourceAggregateDocument(
                            data_source_document=[
                                DataSourceDocumentItem(
                                    data_source_identifier="a1453d7a-dd6b-4630-923f-7e4a14a7ee70",
                                    data_source_feature="cycle threshold result",
                                )
                            ]
                        ),
                        data_processing_document=DataProcessingDocument1(
                            reference_DNA_description="RNaseP",
                            reference_sample_description="800",
                        ),
                        calculated_data_name="ct sd",
                        calculated_data_description=None,
                        calculated_datum=TQuantityValueUnitless(
                            value=0.051,
                            unit="(unitless)",
                            has_statistic_datum_role=None,
                            field_type=None,
                        ),
                    )
                ]
            ),
        ),
        manifest="http://purl.allotrope.org/manifests/pcr/BENCHLING/2023/09/qpcr.manifest",
    )



import pandas as pd
import pytest

from allotropy.allotrope.allotrope import AllotropeConversionError
from allotropy.allotrope.models.shared.definitions.custom import (
    TNullableQuantityValueMilliOsmolesPerKilogram,
)
from allotropy.parsers.roche_cedex_bioht.roche_cedex_bioht_structure import (
    Analyte,
    AnalyteList,
    Data,
    Sample,
    Title,
)
from tests.parsers.roche_cedex_bioht.roche_cedex_bioht_data import get_data, get_reader


@pytest.mark.parametrize(
    "processing_time,model_number,device_serial,analyst",
    [
        ("2021-06-01 13:04:06", "CEDEX BIO HT", 12345, "Analyst1"),
        ("2022-06-01 00:10:06", "CEDEX BIO HT", 99999, "Analyst2"),
        ("2023-06-01 24:35:25", "CEDEX BIO HT", 00000, "Analyst3"),
    ],
)
@pytest.mark.short
def test_create_title(
    processing_time: str, model_number: str, device_serial: int, analyst: str
) -> None:
    title_data = {
        "data processing time": processing_time,
        "model number": model_number,
        "device serial number": device_serial,
        "analyst": analyst,
    }
    title = Title.create(pd.Series(title_data))

    assert title.data_processing_time == processing_time
    assert title.analyst == analyst
    assert title.model_number == model_number
    assert title.device_serial_number == str(device_serial)


@pytest.mark.short
def test_create_title_with_no_analyst() -> None:
    title_data = {"device serial number": 1234}
    with pytest.raises(
        AllotropeConversionError, match="Unable to obtain analyst from input data"
    ):
        Title.create(pd.Series(title_data))


@pytest.mark.short
def test_create_title_with_no_serial_number() -> None:
    title_data = {"analyst": "dummy"}
    with pytest.raises(
        AllotropeConversionError,
        match="Unable to obtain device serial number from input data",
    ):
        Title.create(pd.Series(title_data))


@pytest.mark.short
def test_create_analyte() -> None:
    data = pd.Series(
        {
            "analyte name": "glutamine",
            "concentration value": 2.45,
            "concentration unit": "mmol/L",
        }
    )
    analyte = Analyte.create(data)
    assert analyte.name == "glutamine"
    assert analyte.concentration_value == 2.45
    assert analyte.unit == "mmol/L"


@pytest.mark.short
def test_create_analyte_with_no_unit() -> None:
    data = pd.Series(
        {
            "analyte name": "aspartate",
            "concentration value": 1.45,
        }
    )
    analyte = Analyte.create(data)
    assert analyte.name == "aspartate"
    assert analyte.concentration_value == 1.45
    assert analyte.unit is None


@pytest.mark.short
def test_create_analyte_list() -> None:
    data = pd.DataFrame(
        {
            "analyte name": ["lactate", "glutamine", "osmolality"],
            "concentration unit": ["g/L", "mmol/L", "mosm/kg"],
            "concentration value": [2.45, 4.35, 3.7448],
        }
    )
    analyte_list = AnalyteList.create(data)

    assert analyte_list.analytes == [
        Analyte("lactate", 2.45, "g/L"),
        Analyte("glutamine", 4.35, "mmol/L"),
        Analyte("osmolality", 3.7448, "mosm/kg"),
    ]
    assert analyte_list.non_aggregrable_dict == {
        "osmolality": [TNullableQuantityValueMilliOsmolesPerKilogram(3.7448)]
    }
    assert analyte_list.num_measurement_docs == 1


@pytest.mark.short
def test_create_analyte_list_more_than_one_mesurement_docs() -> None:
    data = pd.DataFrame(
        {
            "analyte name": ["lactate", "glutamine", "glutamine"],
            "concentration unit": ["g/L", "mmol/L", "mmol/L"],
            "concentration value": [2.45, 4.35, 3.45],
        },
    )
    analyte_list = AnalyteList.create(data)

    assert analyte_list.analytes == [
        Analyte("lactate", 2.45, "g/L"),
        Analyte("glutamine", 4.35, "mmol/L"),
        Analyte("glutamine", 3.45, "mmol/L"),
    ]
    assert analyte_list.num_measurement_docs == 2
    assert analyte_list.non_aggregrable_dict == {}


@pytest.mark.short
def test_create_sample() -> None:
    samples_data = pd.DataFrame(
        {
            "sample identifier": ["PPDTEST1", "PPDTEST1", "PPDTEST2"],
            "batch identifier": ["batch_id", "batch_id", ""],
            "analyte name": ["lactate", "glutamine", "glutamine"],
            "concentration unit": ["g/L", "mmol/L", "mmol/L"],
            "concentration value": [2.45, 4.35, 0.4],
            "sample role type": ["Sample", "Sample", "Sample"],
            "measurement time": [
                "2021-05-20 16:55:51",
                "2021-05-20 16:56:51",
                "2023-05-20 16:55:51",
            ],
        }
    )
    sample = Sample.create(
        name="PPDTEST1",
        batch="batch_id",
        samples_data=samples_data,
    )
    assert sample.name == "PPDTEST1"
    assert sample.batch == "batch_id"
    assert sample.role_type == "Sample"
    assert sample.measurement_time == "2021-05-20 16:56:51"
    assert len(sample.analyte_list.analytes) == 2


@pytest.mark.short
def test_create_data() -> None:
    assert Data.create(get_reader()) == get_data()



import pytest

from allotropy.parser_factory import Vendor
from allotropy.parsers.novabio_flex2.novabio_flex2_parser import NovaBioFlexParser
from allotropy.parsers.utils.timestamp_parser import TimestampParser
from tests.parsers.novabio_flex2.novabio_flex2_data import get_data, get_model
from tests.parsers.test_utils import from_file, validate_contents, validate_schema

output_files = (
    "SampleResultsDEVICE1232021-02-18_104838",
    "SampleResults2022-06-28_142558",
)

VENDOR_TYPE = Vendor.NOVABIO_FLEX2


@pytest.mark.parametrize("output_file", output_files)
def test_parse_novabio_flex_to_asm(output_file: str) -> None:
    test_filepath = f"tests/parsers/novabio_flex2/testdata/{output_file}.csv"
    expected_filepath = f"tests/parsers/novabio_flex2/testdata/{output_file}.json"
    allotrope_dict = from_file(test_filepath, VENDOR_TYPE)
    validate_schema(
        allotrope_dict,
        "cell-culture-analyzer/BENCHLING/2023/09/cell-culture-analyzer.json",
    )
    validate_contents(allotrope_dict, expected_filepath)


@pytest.mark.short
def test_get_model() -> None:
    parser = NovaBioFlexParser(TimestampParser())
    model = parser._get_model(get_data())

    if model.measurement_aggregate_document:
        model.measurement_aggregate_document.measurement_identifier = ""

    assert model == get_model()



import io

import numpy as np
import pandas as pd


def convert_datetime(x: pd.Series) -> pd.Series:
    return pd.to_datetime(x).dt.strftime("%Y-%m-%d %H:%M:%S")


def convert_float(x: pd.Series) -> pd.Series:
    return pd.to_numeric(x, errors="coerce")


def convert_int(x: pd.Series) -> pd.Series:
    return pd.to_numeric(x, errors="coerce")


def convert_string(x: pd.Series) -> pd.Series:
    return x.astype(str)


conversors = {
    np.datetime64: convert_datetime,
    np.float64: convert_float,
    np.int64: convert_int,
    np.dtypes.ObjectDType: convert_string,
}


desired_columns = {
    "Analysis by": np.dtypes.ObjectDType,
    "Sample ID": np.dtypes.ObjectDType,
    "Minimum Diameter (μm)": np.float64,
    "Maximum Diameter (μm)": np.float64,
    "Analysis date/time": np.datetime64,
    "Cell type": np.dtypes.ObjectDType,
    "Dilution": np.float64,
    "Viability (%)": np.float64,
    "Total (x10^6) cells/mL": np.float64,
    "Viable (x10^6) cells/mL": np.float64,
    "Average diameter (μm)": np.float64,
    "Average viable diameter (μm)": np.float64,
    "Cell count": np.int64,
    "Viable cells": np.int64,
    "Average circularity": np.float64,
    "Average viable circularity": np.float64,
}


class ViCellBluReader:
    @classmethod
    def read(cls, contents: io.IOBase) -> pd.DataFrame:
        raw_data = pd.read_csv(contents, index_col=False)  # type: ignore[call-overload]

        columns: list[pd.Series] = []
        for column, desired_type in desired_columns.items():
            col = raw_data.get(column)
            if not isinstance(col, pd.Series):
                continue

            new_col = (
                col if col.dtype == desired_type else conversors[desired_type](col)
            )
            columns.append(new_col)

        return pd.concat(columns, axis=1).replace(np.nan, None)



from allotropy.allotrope.allotrope import AllotropeConversionError
from allotropy.parsers.moldev_softmax_pro.absorbance_plate_block import (
    AbsorbancePlateBlock,
)
from allotropy.parsers.moldev_softmax_pro.fluorescence_plate_block import (
    FluorescencePlateBlock,
)
from allotropy.parsers.moldev_softmax_pro.luminescence_plate_block import (
    LuminescencePlateBlock,
)
from allotropy.parsers.moldev_softmax_pro.plate_block import (
    Block,
    GroupBlock,
    NoteBlock,
    PlateBlock,
)
from allotropy.parsers.utils.values import value_or_none

DELIMITER = "\t"


def create_plate_block(raw_lines: list[str]) -> PlateBlock:
    split_lines = [
        [value_or_none(value) for value in raw_line.split(DELIMITER)]
        for raw_line in raw_lines
    ]
    header = split_lines[0]
    read_mode = header[5]
    plate_block_cls: type[PlateBlock]
    if read_mode == "Absorbance":
        plate_block_cls = AbsorbancePlateBlock
    elif read_mode == "Fluorescence":
        plate_block_cls = FluorescencePlateBlock
    elif read_mode == "Luminescence":
        plate_block_cls = LuminescencePlateBlock
    else:
        error = f"unrecognized read mode {read_mode}"
        raise AllotropeConversionError(error)
    plate_block = plate_block_cls(header, split_lines, raw_lines)
    return plate_block


ALL_BLOCKS: list[type[Block]] = [GroupBlock, NoteBlock, PlateBlock]
BLOCK_CLS_BY_TYPE = {cls.BLOCK_TYPE: cls for cls in ALL_BLOCKS}


def create_block(lines: list[str]) -> Block:
    for key, block_cls in BLOCK_CLS_BY_TYPE.items():
        if lines[0].startswith(key):
            if block_cls == PlateBlock:
                return create_plate_block(lines)
            return block_cls(lines)
    error = f"unrecognized block {lines[0]}"
    raise AllotropeConversionError(error)



from typing import Optional

from allotropy.allotrope.allotrope import AllotropeConversionError
from allotropy.parsers.agilent_gen5.absorbance_plate_data import AbsorbancePlateData
from allotropy.parsers.agilent_gen5.constants import ReadMode
from allotropy.parsers.agilent_gen5.fluorescence_plate_data import FluorescencePlateData
from allotropy.parsers.agilent_gen5.luminescence_plate_data import LuminescencePlateData
from allotropy.parsers.agilent_gen5.plate_data import PlateData
from allotropy.parsers.utils.values import assert_not_none


def create_plate_data(
    software_version_chunk: Optional[str],
    file_paths_chunk: Optional[str],
    all_data_chunk: Optional[str],
) -> PlateData:
    software_version_chunk = assert_not_none(
        software_version_chunk, "software_version_chunk"
    )
    file_paths_chunk = assert_not_none(file_paths_chunk, "file_paths_chunk")
    all_data_chunk = assert_not_none(all_data_chunk, "all_data_chunk")

    all_data_sections = all_data_chunk.split("\n\n")
    for data_section in all_data_sections:
        if data_section.startswith("Plate Type"):
            if ReadMode.ABSORBANCE.value in data_section:
                return AbsorbancePlateData(
                    software_version_chunk, file_paths_chunk, all_data_chunk
                )
            elif ReadMode.FLUORESCENCE.value in data_section:
                return FluorescencePlateData(
                    software_version_chunk, file_paths_chunk, all_data_chunk
                )
            elif ReadMode.LUMINESCENCE.value in data_section:
                return LuminescencePlateData(
                    software_version_chunk, file_paths_chunk, all_data_chunk
                )
    msg = "Read mode not found"
    raise AllotropeConversionError(msg)



import pytest

from allotropy.parser_factory import Vendor
from allotropy.parsers.roche_cedex_bioht.roche_cedex_bioht_parser import (
    RocheCedexBiohtParser,
)
from allotropy.parsers.utils.timestamp_parser import TimestampParser
from tests.parsers.roche_cedex_bioht.roche_cedex_bioht_data import get_data, get_model
from tests.parsers.test_utils import from_file, validate_contents, validate_schema

output_files = (
    "roche_cedex_bioht_example01",
    "roche_cedex_bioht_example02",
    "roche_cedex_bioht_example03",
    "roche_cedex_bioht_example04",
)

VENDOR_TYPE = Vendor.ROCHE_CEDEX_BIOHT
SCHEMA_FILE = "cell-culture-analyzer/BENCHLING/2023/09/cell-culture-analyzer.json"


@pytest.mark.parametrize("output_file", output_files)
def test_parse_cedex_bioht_to_asm(output_file: str) -> None:
    test_filepath = f"tests/parsers/roche_cedex_bioht/testdata/{output_file}.txt"
    expected_filepath = f"tests/parsers/roche_cedex_bioht/testdata/{output_file}.json"
    allotrope_dict = from_file(test_filepath, VENDOR_TYPE)
    validate_schema(allotrope_dict, SCHEMA_FILE)
    validate_contents(allotrope_dict, expected_filepath)


@pytest.mark.short
def test_get_model() -> None:
    parser = RocheCedexBiohtParser(TimestampParser())
    model = parser._get_model(get_data())

    if model.measurement_aggregate_document:
        model.measurement_aggregate_document.measurement_identifier = ""

    assert model == get_model()
